local frame = CreateFrame("FRAME", "EveningWearFrame")
frame:RegisterEvent("PLAYER_UPDATE_RESTING")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:RegisterEvent("EQUIPMENT_SWAP_PENDING")
frame:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND")
frame:RegisterEvent("PLAYER_ALIVE")

-- enables debug messages, change to true if you love spam
local debug = false

local notEnoughSpace = "[Evening Wear] |cffff0000ERROR: |rEvening Wear requires two free equipment set slots to operate. Please make space and relog or reloadui."


-- checks to see if we are currently in a major city
local function isMajorCity()
	local myLoc = GetRealZoneText()

	if debug then print("[Evening Wear] Currently in " .. myLoc) end

	if myLoc == "Orgrimmar"
	 	or myLoc == "Shrine of Two Moons"
		or myLoc == "Dalaran"
		or myLoc == "Shattrath City"
		or myLoc == "Stormwind City"
		or myLoc == "Shrine of Seven Stars"
		or myLoc == "Darnassus"
		or myLoc == "The Exodar"
		or myLoc == "Thunder Bluff"
		or myLoc == "Undercity"
		or myLoc == "Silvermoon City"
		or myLoc == "City of Ironforge"
	then return true
	else return false
	end
end

local function ewSwapSets()
	if GetEquipmentSetInfoByName("EveningWear") and GetEquipmentSetInfoByName("Last Equipped") then

		local icon, setID, isEquipped = GetEquipmentSetInfoByName("Evening Wear")
		-- equip Evening Wear or Last Equipped based on whether you're coming or going
		if IsResting() and isMajorCity() then
			if debug then print("[Evening Wear] Player is now resting, equipping Evening Wear set.") end
			
			-- make sure we don't overwrite Last Equipped with Evening Wear
			if isEquipped then
				if debug then print("[Evening Wear] Switching from Evening Wear set, not overwriting Last Equipped.") end
			else
				if debug then print("[Evening Wear] Not switching from Evening Wear set, saving Last Equipped") end
				SaveEquipmentSet("Last Equipped")
			end
	
			UseEquipmentSet("Evening Wear")
		else
			if isEquipped then
				if debug then print("[Evening Wear] Player is no longer resting, equipping Last Equipped set.") end
				UseEquipmentSet("Last Equipped")
			else
				if debug then print("[Evening Wear] Evening Wear set was not equipped, not switching to Last Equipped.") end
			end
		end
	end
end

-- hook the Logout function to swap gear before logging out
local originalLogout = Logout

Logout = function(...)
	local icon, setID, isEquipped = GetEquipmentSetInfoByName("Evening Wear")

	if isEquipped then
		UseEquipmentSet("Last Equipped")
	end
	return originalLogout(...)
end

-- also hook Quit for the same reason
local originalQuit = Quit

Quit = function(...)
	local icon, setID, isEquipped = GetEquipmentSetInfoByName("Evening Wear")

	if isEquipped then
		UseEquipmentSet("Last Equipped")
	end
	return originalQuit(...)
end

local function eventHandler(self, event, ...)
	if event == "PLAYER_ENTERING_WORLD" then
		-- Checks to see if Evening Wear and Last Equipped sets exist, and creates them if not, as long as there is space
		if not GetEquipmentSetInfoByName("Evening Wear") and not GetEquipmentSetInfoByName("Last Equipped") then

			if GetNumEquipmentSets() > 8 then
				print(notEnoughSpace)
			else

				if GetEquipmentSetInfoByName("Evening Wear") == nil and GetNumEquipmentSets() < 10 then
					if debug then print("[Evening Wear] No Evening Wear set existed, creating one") end
					EquipmentManagerIgnoreSlotForSave(2)
					EquipmentManagerIgnoreSlotForSave(11)
					EquipmentManagerIgnoreSlotForSave(12)
					EquipmentManagerIgnoreSlotForSave(13)
					EquipmentManagerIgnoreSlotForSave(14)
					SaveEquipmentSet("Evening Wear", 132665)
				elseif GetNumEquipmentSets() > 9 then
					print(notEnoughSpace)
				end

				if GetEquipmentSetInfoByName("Last Equipped") == nil and GetNumEquipmentSets() < 10 then
					if debug then print("[Evening Wear] No Last Equipped set existed, creating one") end
					SaveEquipmentSet("Last Equipped", 133070)
				elseif GetNumEquipmentSets() > 9 then
					print(notEnoughSpace)
				end
		
				-- Equip the proper set, if not already equipped
				ewSwapSets()
			end
		end

	elseif event == "EQUIPMENT_SWAP_PENDING" then
		-- When an equipment set is changed, for any reason, checks to see if you're swapping from the Evening Wear set. If not, it rewrites Last Equipped. 
		-- This SHOULD mean that when you're messing with gear sets in town and then switch to your Evening Wear when you're done, it remembers what you last had on when you leave.
		local icon, setID, isEquipped = GetEquipmentSetInfoByName("Evening Wear")
		
		if isEquipped then
			if debug then print("[Evening Wear] Switching from Evening Wear set, not overwriting Last Equipped.") end
		else
			if debug then print("[Evening Wear] Not switching from Evening Wear set, saving Last Equipped") end
			SaveEquipmentSet("Last Equipped")
		end

	elseif event == "PLAYER_UPDATE_RESTING" then
		ewSwapSets()

	elseif event == "PLAYER_ENTERING_BATTLEGROUND" then
		ewSwapSets()

	end
end
frame:SetScript("OnEvent", eventHandler)

--[[
--Slash command code, not currently used
SLASH_EVENINGWEAR1, SLASH_EVENINGWEAR2, SLASH_EVENINGWEAR3 = '/ew', '/evw', '/ewear'
function SlashCmdList.EVENINGWEAR(msg, editBox)
	local command, rest = msg:match("^(%S*)%s*(.-)$")
end
--]]

-- onUpdate handler for first login
local hasFired = false

local function onUpdate(self,elapsed)
	if not hasFired and GetRealZoneText() then
		hasFired = true
		ewSwapSets()
	end
end

local f = CreateFrame("frame")
f:SetScript("OnUpdate", onUpdate)